package tle.searchutil;

import java.util.Collection;

import tle.searchutil.impl.CondAnd;
import tle.searchutil.impl.CondBetween;
import tle.searchutil.impl.CondEmpty;
import tle.searchutil.impl.CondEq;
import tle.searchutil.impl.CondEqProperty;
import tle.searchutil.impl.CondGe;
import tle.searchutil.impl.CondGeProperty;
import tle.searchutil.impl.CondGt;
import tle.searchutil.impl.CondGtProperty;
import tle.searchutil.impl.CondIdEq;
import tle.searchutil.impl.CondIlike;
import tle.searchutil.impl.CondIn;
import tle.searchutil.impl.CondLe;
import tle.searchutil.impl.CondLeProperty;
import tle.searchutil.impl.CondLike;
import tle.searchutil.impl.CondLt;
import tle.searchutil.impl.CondLtProperty;
import tle.searchutil.impl.CondNe;
import tle.searchutil.impl.CondNeProperty;
import tle.searchutil.impl.CondNot;
import tle.searchutil.impl.CondNotEmpty;
import tle.searchutil.impl.CondNotIn;
import tle.searchutil.impl.CondNotNull;
import tle.searchutil.impl.CondNull;
import tle.searchutil.impl.CondOr;
import tle.searchutil.impl.OrderImpl;

/**
 * SearchFilter ϵ ˻    Ѵ.
 * @author ֹ
 */
public class Restriction {
	public static PropertyValueCond eq(String propertyName, Object value) {
		return new CondEq(propertyName, value);
	}
	public static PropertyValueCond eq(String propertyName, byte value) {
		return new CondEq(propertyName, new Byte(value));
	}
	public static PropertyValueCond eq(String propertyName, short value) {
		return new CondEq(propertyName, new Short(value));
	}
	public static PropertyValueCond eq(String propertyName, int value) {
		return new CondEq(propertyName, new Integer(value));
	}
	public static PropertyValueCond eq(String propertyName, long value) {
		return new CondEq(propertyName, new Long(value));
	}
	public static PropertyValueCond eq(String propertyName, float value) {
		return new CondEq(propertyName, new Float(value));
	}
	public static PropertyValueCond eq(String propertyName, double value) {
		return new CondEq(propertyName, new Double(value));
	}

	public static PropertyValueCond idEq(Object value) {
		return new CondIdEq(value);
	}
	
	public static TwoPropertyCond eqProperty(String propertyName, String otherPropertyName) {
		return new CondEqProperty(propertyName, otherPropertyName);
	}
	
	public static PropertyValueCond ne(String propertyName, Object value) {
		return new CondNe(propertyName, value);
	}
	public static TwoPropertyCond neProperty(String propertyName, String otherPropertyName) {
		return new CondNeProperty(propertyName, otherPropertyName);
	}

	
	public static PropertyValueCond ge(String propertyName, Object value) {
		return new CondGe(propertyName, value);
	}
	
	public static TwoPropertyCond geProperty(String propertyName, String otherPropertyName) {
		return new CondGeProperty(propertyName, otherPropertyName);
	}
	public static PropertyValueCond gt(String propertyName, Object value) {
		return new CondGt(propertyName, value);
	}
	public static TwoPropertyCond gtProperty(String propertyName, String otherPropertyName) {
		return new CondGtProperty(propertyName, otherPropertyName);
	}
	public static PropertyValueCond le(String propertyName, Object value) {
		return new CondLe(propertyName, value);
	}
	public static TwoPropertyCond leProperty(String propertyName, String otherPropertyName) {
		return new CondLeProperty(propertyName, otherPropertyName);
	}
	public static PropertyValueCond lt(String propertyName, Object value) {
		return new CondLt(propertyName, value);
	}
	
	public static TwoPropertyCond ltProperty(String propertyName, String otherPropertyName) {
		return new CondLtProperty(propertyName, otherPropertyName);
	}
	
	public static PropertyValuesCond between(String propertyName, Object lo, Object hi) {
		return new CondBetween(propertyName, lo, hi);
	}
	
	public static PropertyValuesCond in(String propertyName, Collection values) {
		return new CondIn(propertyName, values);
	}
	public static PropertyValuesCond in(String propertyName, Object[] values) {
		return new CondIn(propertyName, values);
	}
	public static PropertyValuesCond notIn(String propertyName, Collection values) {
		return new CondNotIn(propertyName, values);
	}
	public static PropertyValuesCond notIn(String propertyName, Object[] values) {
		return new CondNotIn(propertyName, values);
	}
	
	public static PropertyValueCond ilike(String propertyName, String value) {
		return new CondIlike(propertyName, value, Match.ANYWHERE);
	}
	public static PropertyValueCond ilike(String propertyName, String value, int matchMode) {
		return new CondIlike(propertyName, value, matchMode);
	}
	public static PropertyValueCond like(String propertyName, String value) {
		return new CondLike(propertyName, value, Match.ANYWHERE);
	}
	public static PropertyValueCond like(String propertyName, String value, int matchMode) {
		return new CondLike(propertyName, value, matchMode);
	}

	public static PropertyCond isEmpty(String propertyName) {
		return new CondEmpty(propertyName);
	}
	public static PropertyCond isNotEmpty(String propertyName) {
		return new CondNotEmpty(propertyName);
	}
	public static PropertyCond isNotNull(String propertyName) {
		return new CondNotNull(propertyName);
	}
	public static PropertyCond isNull(String propertyName) {
		return new CondNull(propertyName);
	}
	
	public static CompositeCond and(Cond lhs, Cond rhs) {
		return new CondAnd().addCond(lhs).addCond(rhs);
	}
	public static CompositeCond or(Cond lhs, Cond rhs) {
		return new CondOr().addCond(lhs).addCond(rhs);
	}
	public static CompositeCond and() {
		return new CondAnd();
	}
	public static CompositeCond or() {
		return new CondOr();
	}
	
	public static SingleCond not(PropertyCond cond) {
		return new CondNot(cond);
	}
	public static SingleCond not(CompositeCond cond) {
		return new CondNot(cond);
	}
	
	public static Order order(String propertyName, boolean asc) {
		return new OrderImpl(propertyName, asc);
	}
}
